import "dart:async";
import "dart:io" as io;

import "package:logging/logging.dart";
import "package:eventsource/publisher.dart";
import "package:eventsource/io_server.dart";

void main() {
  hierarchicalLoggingEnabled = true;
  Logger.root.level = Level.ALL;
  Logger.root.onRecord.listen(print);

  EventSourcePublisher publisher = new EventSourcePublisher(cacheCapacity: 100);

  // start generating dummy events
  generateEvents(publisher);

  // then we use io.HttpServer to serve pages

  var requestHandler = createIoHandler(publisher, gzip: false);

  io.HttpServer.bind("localhost", 8080).then((io.HttpServer server) {
    // or we can first filter on path
    server.listen((io.HttpRequest request) {
      print(request);
      if (request.uri.path == "/events") {
        requestHandler(request);
      }
    });
  });
}

generateEvents(EventSourcePublisher publisher) {
  int id = 0;
  new Timer.periodic(const Duration(seconds: 3), (timer) {
    if (id == 5) {
      timer.cancel();
      // broadcast last message to both channels and close them
      publisher
          .add(new Event(id: "$id", event: "goodbye", data: "Goodbye all!"));
      publisher.closeAllChannels();
      return;
    }
    // publish an event on the default channel
    publisher
        .add(new Event.message(id: "$id", data: "Always the same message?"));
    id++;
  });
}
